<?php
namespace Drupal\commerce_rbspayment_alfabank\Plugin\Commerce\PaymentGateway;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\commerce_payment\PaymentMethodTypeManager;
use Drupal\commerce_payment\PaymentTypeManager;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\commerce_price\Entity\Currency;
use Drupal\commerce_price\Price;
use Drupal\commerce_price\RounderInterface;
use Drupal\commerce_rbspayment_alfabank\CommerceRBSPaymentAlfabankApi;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Component\Utility\Unicode;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
/**
* Provides the Off-site Gateway payment.
*
* @CommercePaymentGateway(
*   id = "commerce_rbspayment_alfabank",
*   label = "RBSPaymentAlfabank",
*   display_label = "Bank Card",
*   forms = {
*     "offsite-payment" = "Drupal\commerce_rbspayment_alfabank\PluginForm\OffsiteRedirect\PaymentOffsiteForm",
*   },
*   payment_method_types = {"credit_card"},
*   credit_card_types = {
*     "amex", "jcb", "unionpay", "mastercard", "visa",
*   },
* )
*/
require_once DRUPAL_ROOT ."/". \Drupal::service('extension.list.module')->getPath('commerce_rbspayment_alfabank') ."/src/cfg/include.php";
class RBSPaymentAlfabank extends OffsitePaymentGatewayBase implements RBSPaymentAlfabankInterface
{
/**
* The price rounder.
*
* @var \Drupal\commerce_price\RounderInterface
*/
protected $rounder;
public function __construct(array $configuration,
$plugin_id,
$plugin_definition,
EntityTypeManagerInterface $entity_type_manager,
PaymentTypeManager $payment_type_manager,
PaymentMethodTypeManager $payment_method_type_manager,
TimeInterface $time,
RounderInterface $rounder)
{
parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $payment_type_manager, $payment_method_type_manager, $time);
$this->rounder = $rounder;
}
/**
* {@inheritdoc}
*/
public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition)
{
return new static(
$configuration,
$plugin_id,
$plugin_definition,
$container->get('entity_type.manager'),
$container->get('plugin.manager.commerce_payment_type'),
$container->get('plugin.manager.commerce_payment_method_type'),
$container->get('datetime.time'),
$container->get('commerce_price.rounder')
);
}
public function capturePayment(PaymentInterface $payment, Price $amount = NULL)
{
$this->assertPaymentState($payment, ['authorization']);
$amount = $amount ?: $payment->getAmount();
try {
$remote_id = $payment->getRemoteId();
$decimal_amount = $amount->getNumber() * 100;
$result = $this->getApi($payment)->confirmOrderPayment($remote_id, $decimal_amount);
if ($result['errorCode'] > 0) {
throw new PaymentGatewayException("Could not capture payment. Message: " .$result['errorMessage']);
}
$payment->setState('completed');
$payment->setAmount($amount);
$payment->save();
}
catch (\PaymentGatewayException $e) {
throw new PaymentGatewayException('capturePayment Error ', $e);
}
}
public function refundPayment(PaymentInterface $payment, Price $amount = NULL)
{
$this->assertPaymentState($payment, ['completed', 'partially_refunded']);
$amount = $amount ?: $payment->getAmount();
$this->assertRefundAmount($payment, $amount);
try {
$remote_id = $payment->getRemoteId();
$decimal_amount = $amount->getNumber() * 100;
$result = $this->getApi($payment)->refundOrderPayment($remote_id, $decimal_amount);
if ($result['errorCode'] > 0) {
throw new PaymentGatewayException("Could not refund payment. Message: " .$result['errorMessage']);
}
$old_refunded_amount = $payment->getRefundedAmount();
$new_refunded_amount = $old_refunded_amount->add($amount);
if ($new_refunded_amount->lessThan($payment->getAmount())) {
$payment->setState('partially_refunded');
}
else {
$payment->setState('refunded');
}
$payment->setRefundedAmount($new_refunded_amount);
$payment->save();
}
catch (\PaymentGatewayException $e) {
throw new PaymentGatewayException('refundPayment Error ', $e);
}
}
/**
* {@inheritdoc}
*/
public function voidPayment(PaymentInterface $payment)
{
$this->assertPaymentState($payment, ['authorization']);
try {
$remote_id = $payment->getRemoteId();
$result = $this->getApi($payment)->reverseOrderPayment($remote_id);
if ($result['errorCode'] > 0) {
throw new PaymentGatewayException("Could not void payment. Message: " . $result['errorMessage']);
}
$payment->setState('authorization_voided');
$payment->save();
}
catch (\PaymentGatewayException $e) {
throw new PaymentGatewayException('voidPayment Error ', $e);
}
}
/**
* {@inheritdoc}
*/
public function defaultConfiguration()
{
return [
'username' => '',
'password' => '',
'token' => '',
'double_staged' => false,
'enable_cacert' => true,
'backToShopUrl' => null,
'backToShopUrlName' => null,
'logging' => false,
'send_cart_data' => false,
'versionFfd' => 'v1_05',
'taxSystem' => 0,
'paymentMethodType' => 1,
'paymentObjectType' => 1,
] + parent::defaultConfiguration();
}
/**
* {@inheritdoc}
*/
public function buildConfigurationForm(array $form, FormStateInterface $form_state)
{
$form = parent::buildConfigurationForm($form, $form_state);
$default_login = $this->configuration['username'];
$default_password = $this->configuration['password'];
$token_default = '';
if (!empty($default_login) && !empty($default_password)) {
$default_token = base64_encode($default_login . ":" . $default_password);
}
if (defined('RBSPAYMENT_ALFABANK_API_VERSION') && RBSPAYMENT_ALFABANK_API_VERSION == 2) {
$form['token'] = [
'#type' => 'textfield',
'#title' => $this->t('Token'),
'#default_value' => !empty($this->configuration['token']) ? $this->configuration['token'] : $default_token,
'#required' => TRUE,
'#attributes' => [
'id' => 'idd-token',
],
];
} else  {
$form['username'] = [
'#type' => 'textfield',
'#title' => $this->t('Login-API'),
'#default_value' => $default_login,
'#required' => TRUE,
];
$form['password'] = [
'#type' => 'textfield',
'#title' => $this->t('Password'),
'#default_value' => $default_password,
'#required' => TRUE,
];
}
$form['double_staged'] = [
'#type' => 'checkbox',
'#title' => $this->t('Two-phase payments'),
'#default_value' => $this->configuration['double_staged'],
];
if (file_exists(DRUPAL_ROOT ."/". \Drupal::service('extension.list.module')->getPath('commerce_rbspayment_alfabank') ."/src/cfg/cacert.cer")) {
$form['enable_cacert'] = [
'#type' => 'checkbox',
'#title' => $this->t('Verify SSL certificate '),
'#default_value' => $this->configuration['enable_cacert'],
];
}
if (defined('RBSPAYMENT_ALFABANK_ENABLE_BACK_URL_SETTINGS') && RBSPAYMENT_ALFABANK_ENABLE_BACK_URL_SETTINGS == true) {
$form['backToShopUrl'] = [
'#type' => 'textfield',
'#title' => $this->t('Back to shop URL'),
'#default_value' => $this->configuration['backToShopUrl'],
'#required' => false,
];
}
if (defined('RBSPAYMENT_ALFABANK_ENABLE_CART_OPTIONS') && RBSPAYMENT_ALFABANK_ENABLE_CART_OPTIONS == true){
$form['send_cart_data'] = [
'#type' => 'checkbox',
'#title' => $this->t('Send cart data (including customer info)'),
'#default_value' => $this->configuration['send_cart_data'],
];
$form['taxSystem'] = array(
'#type' => 'select',
'#title' => $this->t('Tax system'),
'#options' => array(
0 => $this->t('General'),
1 => $this->t('Simplified, income'),
2 => $this->t('Simplified, income minus expences'),
3 => $this->t('Unified tax on imputed income'),
4 => $this->t('Unified agricultural tax'),
5 => $this->t('Patent taxation system'),
),
'#default_value' => $this->configuration['taxSystem'],
'#required' => false,
);
$form['tax_type'] = array(
'#title' => $this->t('Default VAT'),
'#type' => 'select',
'#options' => array(
'0' => $this->t('No VAT'),
'1' => $this->t('VAT 0%'),
'2' => $this->t('VAT 10%'),
'3' => $this->t('VAT 18%'),
'6' => $this->t('VAT 20%'),
'4' => $this->t('VAT applicable rate 10/110'),
'5' => $this->t('VAT applicable rate 18/118'),
'7' => $this->t('VAT applicable rate 20/120'),
),
'#default_value' => $this->configuration['tax_type'],
'#required' => false,
);
$form['versionFfd'] = array(
'#type' => 'select',
'#title' => $this->t('Fiscal document format'),
'#options' => array(
'v1_05' => 'v1.05',
'v1_2' => 'v1.2',
),
'#default_value' => $this->configuration['versionFfd'],
'#required' => false,
);
$form['paymentMethodType'] = array(
'#type' => 'select',
'#title' => $this->t('Payment type'),
'#options' => array(
1 => $this->t('Full prepayment'),
2 => $this->t('Partial prepayment'),
3 => $this->t('Advance payment'),
4 => $this->t('Full payment'),
5 => $this->t('Partial payment with further credit'),
6 => $this->t('No payment with further credit'),
7 => $this->t('Payment on credit'),
),
'#default_value' => $this->configuration['paymentMethodType'],
'#description' => "",
'#required' => false,
);
$form['paymentObjectType'] = array(
'#type' => 'select',
'#title' => $this->t('Type of goods and services'),
'#options' => array(
1 => $this->t('Goods'),
2 => $this->t('Excised goods'),
3 => $this->t('Job'),
4 => $this->t('Service'),
5 => $this->t('Stake in gambling'),
7 => $this->t('Lottery ticket'),
9 => $this->t('Intellectual property provision'),
10 => $this->t('Payment'),
11 => $this->t('Agent\'s commission'),
12 => $this->t('Combined'),
13 => $this->t('Other'),
),
'#default_value' => $this->configuration['paymentObjectType'],
'#description' => "",
'#required' => false,
);
}
$form['logging'] = [
'#type' => 'checkbox',
'#title' => $this->t('Write log-file'),
'#default_value' => $this->configuration['logging'],
];
$form['#attached']['html_head'][] = [
[
'#tag' => 'script',
'#attributes' => [],
'#value' => "
/* IDD_MODULE_VERSION: 2.1.4 */
document.addEventListener('DOMContentLoaded', function() {
var inputElement = document.getElementById('idd-token');
if (inputElement) {
inputElement.type = 'password';
}
});
",
],
'maskFieldInline',
];
return $form;
}
/**
* {@inheritdoc}
*/
public function submitConfigurationForm(array &$form, FormStateInterface $form_state)
{
parent::submitConfigurationForm($form, $form_state);
if (!$form_state->getErrors()) {
$values = $form_state->getValue($form['#parents']);
if (!empty($values['username'])) {
$this->configuration['username'] = $values['username'];
}
if (!empty($values['password'])) {
$this->configuration['password'] = $values['password'];
}
if (!empty($values['token'])) {
$this->configuration['token'] = $values['token'];
}
$this->configuration['double_staged'] = $values['double_staged'];
$this->configuration['logging'] = $values['logging'];
$this->configuration['send_cart_data'] = empty($values['send_cart_data']) ? false : $values['send_cart_data'];;
$this->configuration['enable_cacert'] = empty($values['enable_cacert']) ? null : $values['enable_cacert'];
$this->configuration['versionFfd'] = empty($values['versionFfd']) ? null : $values['versionFfd'];
$this->configuration['taxSystem'] = empty($values['taxSystem']) ? null : $values['taxSystem'];
$this->configuration['tax_type'] = empty($values['tax_type']) ? null : $values['tax_type'];
$this->configuration['paymentMethodType'] = empty($values['paymentMethodType']) ? null : $values['paymentMethodType'];
$this->configuration['paymentObjectType'] = empty($values['paymentObjectType']) ? null : $values['paymentObjectType'];
$this->configuration['backToShopUrl'] = $values['backToShopUrl'];
}
}
/**
* {@inheritdoc}
*/
public function onReturn(OrderInterface $order, Request $request)
{
$callback = $request->query->get('callback');
if (isset($callback)) {
$remote_id = $request->query->get('mdOrder');
} else {
$remote_id = $request->query->get('orderId');
}
if (!isset($remote_id)) {
die("Failed to request!");
}
switch ($this->getMode()) {
default:
case 'test':
$action_adr = RBSPAYMENT_ALFABANK_TEST_URL;
break;
case 'live':
$action_adr = RBSPAYMENT_ALFABANK_PROD_URL;
if (defined('RBSPAYMENT_ALFABANK_PROD_URL_ALTERNATIVE_DOMAIN') && defined('RBSPAYMENT_ALFABANK_PROD_URL_ALT_PREFIX')) {
if (substr($this->configuration['username'], 0, strlen(RBSPAYMENT_ALFABANK_PROD_URL_ALT_PREFIX)) == RBSPAYMENT_ALFABANK_PROD_URL_ALT_PREFIX) {
$pattern = '/^https:\/\/[^\/]+/';
$action_adr = preg_replace($pattern, rtrim(RBSPAYMENT_ALFABANK_PROD_URL_ALTERNATIVE_DOMAIN, '/'), $action_adr);
} else {
}
}
break;
}
$args = array(
'orderId' => $remote_id
);
if (!empty($this->configuration['token'])) {
$decoded_credentials = base64_decode($this->configuration['token']);
list($l, $p) = explode(':', $decoded_credentials);
$args['userName'] = $l;
$args['password'] = $p;
} else {
$args['userName'] = $this->configuration['username'];
$args['password'] = $this->configuration['password'];
}
$ch = curl_init();
curl_setopt_array($ch, array(
CURLOPT_URL => $action_adr . "getOrderStatusExtended.do",
CURLOPT_RETURNTRANSFER => true,
CURLOPT_POST => true,
CURLOPT_SSL_VERIFYPEER => false,
CURLOPT_POSTFIELDS => http_build_query($args, '', '&'),
));
$response = curl_exec($ch);
curl_close($ch);
$response = json_decode($response, true);
$payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
$payment = $payment_storage->loadByRemoteId($remote_id);
$orderStatus = $response['orderStatus'];
if ($orderStatus == '1' || $orderStatus == '2') {
($orderStatus == '2') ? $payment->setState('completed') : $payment->setState('authorization');
$payment->setAmount($order->getTotalPrice());
$payment->setRemoteState("Payed");
$payment->setCompletedTime(time());
$payment->save();
} else {
$payment->setState('authorization_voided');
$payment->save();
throw new PaymentGatewayException('Payment failed!');
}
$this->setLocalState($payment, $orderStatus);
}
/**
* Sets transaction 'status' and 'message' depending on Gateway status.
*
* @param PaymentInterface $payment
* @param int $remote_status
*/
public function setLocalState(PaymentInterface $payment, $remote_status)
{
switch ($remote_status) {
case CommerceRBSPaymentAlfabankApi::orderStatusPending:
$payment->setState('pending');
break;
case CommerceRBSPaymentAlfabankApi::orderStatusPreHold:
$payment->setState('authorization');
break;
case CommerceRBSPaymentAlfabankApi::orderStatusAuthorized:
$payment->setState('completed');
break;
case CommerceRBSPaymentAlfabankApi::orderStatusDeclined:
$payment->setState('authorization_voided');
break;
case CommerceRBSPaymentAlfabankApi::orderStatusPartlyRefunded:
$status = $payment->getBalance()->isZero() ? 'refunded' : 'partially_refunded';
$payment->setState($status);
break;
case CommerceRBSPaymentAlfabankApi::orderStatusReversed:
$payment->setState('refunded');
break;
}
}
/**
* {@inheritdoc}
*/
public function onCancel(OrderInterface $order, Request $request)
{
parent::onCancel($order, $request);
}
/**
* {@inheritdoc}
*/
public function onNotify(Request $request)
{
$data = $request->query->all();
/** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
$payment = $this->entityTypeManager->getStorage('commerce_payment')->load($data['orderNumber']);
if (is_null($payment) || $payment->getRemoteId() != $data['mdOrder']) {
return FALSE;
}
$status_response = $this->getApi($payment)->getOrderStatusByGatewayOrderId($data['mdOrder']);
switch ($data['operation']) {
case 'approved':
break;
case 'deposited':
$status = $this->doDeposited($payment, $status_response);
break;
case 'reversed':
case 'refunded':
$status = $this->doRefunded($payment, $status_response);
break;
case 'declinedByTimeout':
$status = $this->doCancel($payment, $status_response);
break;
}
if (!$status) {
return FALSE;
}
$payment->save();
$payment->getOrder()->setData($this->getPluginId(), $status_response)->save();
}
public function getApi(PaymentInterface $payment)
{
/** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $payment_gateway_plugin */
$payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
$payment_gateway_configuration = $payment_gateway_plugin->getConfiguration();
if (!empty($payment_gateway_configuration['token'])) {
$decoded_credentials = base64_decode($payment_gateway_configuration['token']);
list($l, $p) = explode(':', $decoded_credentials);
$params['userName'] = $l;
$params['password'] = $p;
} else {
$params['userName'] = $payment_gateway_configuration['username'];
$params['password'] = $payment_gateway_configuration['password'];
}
$params['isDoubleStaged'] = $payment_gateway_configuration['double_staged'] == 0 ? false : true;
$params['isTestMode'] = $payment->getPaymentGatewayMode() == 'live' ? false : true;
$params['isLogging'] = $payment_gateway_configuration['logging'] == 0 ? false : true;
$params['enable_cacert'] = $payment_gateway_configuration['enable_cacert'] == 0 ? false : true;
$params['isSendCartData'] = $payment_gateway_configuration['send_cart_data'] == 0 ? false : true;
$params['versionFfd'] = $payment_gateway_configuration['versionFfd'];
$params['taxSystem'] = $payment_gateway_configuration['taxSystem'];
$params['paymentMethodType'] = $payment_gateway_configuration['paymentMethodType'];
$params['paymentObjectType'] = $payment_gateway_configuration['paymentObjectType'];
return new CommerceRBSPaymentAlfabankApi($params);
}
protected function doDeposited(PaymentInterface $payment, array $status_response)
{
$response_amount = intdiv($status_response['amount'], 100);
$gateway_price = new Price($response_amount, $this->getCurrencyCode($status_response['currency']));
if (!$payment->getAmount()->equals($gateway_price)) {
return FALSE;
}
$payment->setRemoteState($status_response['paymentAmountInfo']['paymentState']);
$this->setLocalState($payment, $status_response['orderStatus']);
return TRUE;
}
/**
* @param string $currency_code
* @return string
*/
function getCurrencyCode($numeric_currency_code)
{
/** @var Currency[] $currencies */
$currencies = Currency::loadMultiple();
foreach ($currencies as $currency) {
if ($currency->getNumericCode() == $numeric_currency_code) {
return $currency->getCurrencyCode();
}
}
return NULL;
}
protected function doRefunded(PaymentInterface $payment, array $status_response)
{
$response_amount = intdiv($status_response['paymentAmountInfo']['refundedAmount'], 100);
$refunded_amount = new Price($response_amount, $this->getCurrencyCode($status_response['currency']));
$payment->setRefundedAmount($refunded_amount);
$payment->setRemoteState($status_response['paymentAmountInfo']['paymentState']);
$this->setLocalState($payment, $status_response['orderStatus']);
return TRUE;
}
protected function doCancel(PaymentInterface $payment, array $status_response)
{
$payment->setRemoteState($status_response['paymentAmountInfo']['paymentState']);
$payment->setState('authorization_expired');
return TRUE;
}
/**
* {@inheritdoc}
*/
public function buildPaymentOperations(PaymentInterface $payment)
{
$payment_state = $payment->getState()->value;
$operations = parent::buildPaymentOperations($payment);
return $operations;
}
}
